<?php
/* --------------------------------------------------------------
   ThemeWriter.php 2019-09-12
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2019 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

use Gambio\StyleEdit\Core\Repositories\Entities\ConfigurationCollection;
use League\Flysystem\FileExistsException;

/**
 * Class ThemeWriter
 */
class ThemeWriter implements ThemeWriterInterface
{
    const FILE_IS_SCRIPT_OR_SHEET_PATTERN = '#\.(css|js)$#i';
    
    const UNMINIFIED_FILE_NAME_PATTERN = '#(\.min)?\.(css|js)$#i';
    
    protected $filesystem;
    
    protected $shopSource;
    
    protected $sourceThemeId;
    
    
    /**
     * ThemeWriter constructor.
     *
     * @param FilesystemAdapter $filesystem
     * @param ExistingDirectory $shopSource
     */
    public function __construct(FilesystemAdapter $filesystem, ExistingDirectory $shopSource)
    {
        $this->filesystem = $filesystem;
        $this->shopSource = $shopSource->getAbsolutePath();
    }
    
    
    /**
     * Saves the given them to the destination.
     *
     * @param ThemeInterface              $theme       Theme to be saved.
     * @param ThemeDirectoryRootInterface $destination Destination directory.
     *
     * @return void
     * @throws Exception
     */
    public function save(ThemeInterface $theme, ThemeDirectoryRootInterface $destination)
    {
        $this->sourceThemeId = $theme->getId();
        
        $destinationDirectory = $this->_getRelative($destination);
        $contents             = $this->filesystem->listContents($destinationDirectory);
        
        // delete all files and directories in the theme folder except for the index.html
        foreach ($contents as $content) {
            $path = '/' . $content['path'];
            if ($content['type'] === 'dir') {
                $this->filesystem->deleteDir($path);
            } elseif ($path !== $destinationDirectory . '/index.html') {
                $this->filesystem->delete('/' . $content['path']);
            }
        }
        
        @$this->filesystem->createDir($destinationDirectory, ['visibility' => 'public']);
        
        $childThemes = [];
        
        while ($theme->hasParent()) {
            $childThemes[] = $theme;
            $theme         = $theme->getParentTheme();
        }
        // at this point, $theme is the main theme and $childThemes contains all child themes, reversed.
        
        $childThemes = array_reverse($childThemes);
        $this->_saveThemeStructure($theme->toMainTheme(), $destination, ...$childThemes);
        $this->findFilesThatHaveNoMatchingMinOrSourceFile($destination);
    }
    
    
    /**
     * @param ThemeDirectoryRootInterface $directory
     *
     * @return mixed
     */
    protected function _getRelative(ThemeDirectoryRootInterface $directory)
    {
        return str_replace($this->shopSource, '', $directory->getPath());
    }
    
    
    /**
     * @param MainThemeInterface          $mainTheme
     * @param ThemeDirectoryRootInterface $destination
     * @param ThemeInterface              ...$childThemes
     *
     * @throws FileNotFoundException
     */
    protected function _saveThemeStructure(
        MainThemeInterface $mainTheme,
        ThemeDirectoryRootInterface $destination,
        ThemeInterface ...$childThemes
    ) {
        
        $this->_saveMainTheme($mainTheme, $destination);
        $overloaders = CustomThemeCopyResponse::create();
        $activeTheme = count($childThemes) > 0 ? end($childThemes) : $mainTheme;
        $this->_copySettingsJson($activeTheme, $destination);
        $this->_copyThemeJson($activeTheme, $destination);
        reset($childThemes);
        
        $overloaders->appendWhitelist($this->_applyActiveThemeVariants($mainTheme, $destination));
        
        foreach ($childThemes as $childTheme) {
            $overloaders->append($this->_saveTheme($childTheme, $destination));
            $overloaders->appendWhitelist($this->_applyActiveThemeVariants($childTheme, $destination));
        }
        
        $this->applyHtmlOverloads($overloaders, $destination);
    }
    
    
    /**
     * Checks the $themeDirectoryRoot (/public/theme)
     * for files that have no matching source or .min file
     *
     * @param ThemeDirectoryRootInterface $themeDirectoryRoot
     */
    protected function findFilesThatHaveNoMatchingMinOrSourceFile(ThemeDirectoryRootInterface $themeDirectoryRoot)
    {
        $files = $this->findScriptsAndSheetsInDirectory(new StringType($themeDirectoryRoot->getPath()));
        
        $matchedFiles = [];
        
        // creating an two dimensional array matching .min files with source files
        foreach ($files as $file) {
            
            $notMinifiedFileName = preg_replace(self::UNMINIFIED_FILE_NAME_PATTERN, '.$2', $file);
            
            if (!isset($matchedFiles[$notMinifiedFileName])) {
                
                $matchedFiles[$notMinifiedFileName] = [];
            }
            
            $matchedFiles[$notMinifiedFileName][] = $file;
        }
        
        // detecting files that have no min or source file
        $singularFiles = array_filter($matchedFiles,
            static function ($item) {
                
                return count($item) === 1 ? array_shift($item) : null;
            });
        
        //  creating a one dimensional none associative array
        $singularFiles = array_values(array_map('array_shift', $singularFiles));
        
        $this->duplicateFilesThatHaveNoMatchingMinOrSourceFile($singularFiles);
    }
    
    
    /**
     * @param MainThemeInterface          $mainTheme
     * @param ThemeDirectoryRootInterface $destination
     */
    protected function _saveMainTheme(MainThemeInterface $mainTheme, ThemeDirectoryRootInterface $destination)
    {
        $this->_copyMainThemeDirectory($mainTheme->getId(), $mainTheme->getConfig(), $destination);
        if ($mainTheme->getVariants()) {
            $this->_copyMainThemeDirectory($mainTheme->getId(), $mainTheme->getVariants(), $destination);
        }
        $this->_copyMainThemeDirectory($mainTheme->getId(), $mainTheme->getFonts(), $destination);
        $this->_copyMainThemeDirectory($mainTheme->getId(), $mainTheme->getHtml(), $destination);
        $this->_copyMainThemeDirectory($mainTheme->getId(), $mainTheme->getImages(), $destination);
        if ($mainTheme->getJs()) {
            $this->_copyMainThemeDirectory($mainTheme->getId(), $mainTheme->getJs(), $destination);
        }
        $this->_copyMainThemeDirectory($mainTheme->getId(), $mainTheme->getStyles(), $destination);
        $this->_copyMainThemeStyleEditDirectory($mainTheme->getId(), $mainTheme->getStyleEdit(), $destination);
    }
    
    
    /**
     * @param VariableThemeDirectoriesInterface $theme
     * @param ThemeDirectoryRootInterface       $destination
     *
     * @throws Exception
     */
    protected function _copySettingsJson(
        VariableThemeDirectoriesInterface $theme,
        ThemeDirectoryRootInterface $destination
    ) {
        $jsonSource      = $this->_getRelative($theme->getConfig()->getRoot()) . DIRECTORY_SEPARATOR . '..'
                           . DIRECTORY_SEPARATOR . 'settings.json';
        $jsonDestination = $this->_getRelative($destination) . DIRECTORY_SEPARATOR . 'settings.json';
        
        try {
            if ($this->filesystem->has($jsonSource)) {
                $this->filesystem->copy($jsonSource, $jsonDestination);
            } else {
                if ($this->filesystem->has($jsonDestination)) {
                    $this->filesystem->delete($jsonDestination);
                }
                $this->filesystem->put($jsonDestination, '[]');
            }
        } catch (FileNotFoundException $notFoundException) {
            return;
        }
    }
    
    
    /**
     * @param VariableThemeDirectoriesInterface $theme
     * @param ThemeDirectoryRootInterface       $destination
     *
     * @throws Exception
     */
    protected function _copyThemeJson(
        VariableThemeDirectoriesInterface $theme,
        ThemeDirectoryRootInterface $destination
    ) {
        $jsonSource      = $this->_getRelative($theme->getConfig()->getRoot()) . DIRECTORY_SEPARATOR . '..'
                           . DIRECTORY_SEPARATOR . 'theme.json';
        $jsonDestination = $this->_getRelative($destination) . DIRECTORY_SEPARATOR . 'theme.json';
        
        try {
            $this->filesystem->copy($jsonSource, $jsonDestination);
        } catch (FileNotFoundException $notFoundException) {
            return;
        }
    }
    
    
    /**
     * @param IdentifiedThemeInterface    $theme
     * @param ThemeDirectoryRootInterface $destination
     *
     * @return bool|CustomThemeCopyResponse
     * @throws FileNotFoundException
     */
    protected function _applyActiveThemeVariants(
        IdentifiedThemeInterface $theme,
        ThemeDirectoryRootInterface $destination
    ) {
        $result   = CustomThemeCopyResponse::create();
        $variants = $this->_loadActiveVariants($destination);
        foreach ($variants as $selectedVariant) {
            $variantPath = 'variants/' . $selectedVariant->value()->id;
            if ($theme->getRoot()->hasPath($variantPath)) {
                
                $variant = VariantDirectories::createWithCustomPrefix($theme->getRoot()->withPath($variantPath),
                                                                      $theme->getId());
                $result->appendWhitelist($this->_overloadVariantDirectory($variant, $destination));
            }
        }
        
        return $result;
    }
    
    
    /**
     * @param ThemeInterface              $theme
     * @param ThemeDirectoryRootInterface $destination
     *
     * @return bool|CustomThemeCopyResponse|mixed
     * @throws FileNotFoundException
     */
    protected function _saveTheme(ThemeInterface $theme, ThemeDirectoryRootInterface $destination)
    {
        $this->_copyThemeDirectory($theme->getId(), $theme->getConfig(), $destination);
        $this->_copyThemeDirectory($theme->getId(), $theme->getHtml(), $destination);
        $result = $this->_copyCustomHtmlDirectory($theme, $theme->getCustomHtml(), $destination);
        
        $this->_copyThemeDirectory($theme->getId(), $theme->getJs(), $destination);
        $this->_copyCustomJsDirectory($theme->getId(), $theme->getCustomJs(), $destination);
        
        $this->_copyThemeDirectory($theme->getId(), $theme->getStyles(), $destination);
        $this->_copyCustomStylesDirectory($theme->getId(), $theme->getCustomStyles(), $destination);
        
        $this->_copyThemeImagesDirectory($theme->getId(), $theme->getImages(), $destination);
        
        $this->_copyStyleEditThemeDirectory($theme->getId(), $theme->getStyleEdit(), $destination);
        
        $this->_copyThemeDirectory($theme->getId(), $theme->getJsExtensions(), $destination);
        
        return $result;
    }
    
    
    /**
     * @param CustomThemeCopyResponse     $overloaders
     * @param ThemeDirectoryRootInterface $destination
     *
     * @return bool|CustomThemeCopyResponse
     * @throws FileNotFoundException
     */
    public function applyHtmlOverloads(CustomThemeCopyResponse $overloaders, ThemeDirectoryRootInterface $destination)
    {
        if (count($overloaders->getOverloaders())) {
            
            $destinationPath = $this->_getRelative($destination->withPath('html' . DIRECTORY_SEPARATOR . 'system'));
            $htmlPrefix      = 'a' . md5(microtime(true));
            //save theme overload file
            $htmlThemeOverloadFilename = $htmlPrefix . '.html';
            $htmlThemeOverloadFilePath = $destinationPath . DIRECTORY_SEPARATOR . $htmlThemeOverloadFilename;
            $htmlThemeOverloadContent  = '';
            
            foreach ($overloaders->getOverloaders() as $existingFile) {
                $resource                 = $this->filesystem->readStream($this->getRelativeFromFile($existingFile));
                $htmlThemeOverloadContent .= PHP_EOL . stream_get_contents($resource);
            }
            
            $putStream = tmpfile();
            fwrite($putStream, $htmlThemeOverloadContent);
            rewind($putStream);
            $this->filesystem->putStream($htmlThemeOverloadFilePath, $putStream);
            
            //create temporary file to be use as model to create the final overloads
            $htmlTempFilename = $htmlPrefix . 'temp.html';
            $htmlTempFilePath = $destinationPath . DIRECTORY_SEPARATOR . $htmlTempFilename;
            $htmlTempContent  = '{include file="get_usermod:{$tpl_path}' . $htmlThemeOverloadFilename . '"}';
            
            $putStream = tmpfile();
            fwrite($putStream, $htmlTempContent);
            rewind($putStream);
            $this->filesystem->putStream($htmlTempFilePath, $putStream);
            
            //create whitelist
            $whitelist   = array_map(function (ExistingFile $file) {
                return $this->getRelativeFromFile($file);
            },
                $overloaders->getWhiteList()->getArray());
            $whitelist[] = $htmlThemeOverloadFilePath;
            $whitelist[] = $htmlTempFilePath;
            
            //extends all the files
            $path                 = str_replace($this->_getRelative($destination), '', $destinationPath);
            $destinationDirectory = ThemeDirectory::create($destination->withPath($path));
            foreach ($destinationDirectory->getFiles() as $fileToBeOverloaded) {
                if (strpos($fileToBeOverloaded, 'content_zone_') !== 0) {
                    $filePath = $destinationPath . DIRECTORY_SEPARATOR . $fileToBeOverloaded;
                    if (!in_array($filePath, $whitelist, false)) {
                        $this->_extendHtmlFile($htmlTempFilePath,
                                               $filePath,
                                               array_shift(explode('.', $htmlThemeOverloadFilename)));
                    }
                }
            }
            $this->filesystem->delete($htmlTempFilePath);
            
            return CustomThemeCopyResponse::createWithOverloadsAndWhitelist(new ExistingFileCollection([]),
                                                                            new ExistingFileCollection([
                                                                                                           new ExistingFile(new NonEmptyStringType($this->shopSource
                                                                                                                                                   . DIRECTORY_SEPARATOR
                                                                                                                                                   . $htmlThemeOverloadFilePath))
                                                                                                       ]));
        }
    }
    
    
    /**
     * @param StringType $directory
     *
     * @return string[]
     */
    protected function findScriptsAndSheetsInDirectory(StringType $directory)
    {
        $result = [];
        
        foreach (new DirectoryIterator($directory->asString()) as $file) {
            
            if (!$file->isDot()) {
                
                $fileName = new StringType($file->getPath() . DIRECTORY_SEPARATOR . $file->getFilename());
                
                if ($file->isDir()) {
                    
                    foreach ($this->findScriptsAndSheetsInDirectory($fileName) as $recursivelyFoundFile) {
                        
                        $result[] = $recursivelyFoundFile;
                    }
                } elseif ($file->isFile() && preg_match(self::FILE_IS_SCRIPT_OR_SHEET_PATTERN, $fileName->asString())) {
                    
                    $result[] = $fileName->asString();
                }
            }
        }
        
        return $result;
    }
    
    
    /**
     * Creates a duplicate of a file with .min if only the source file exists
     * or a file without .min if only the .min file exists
     *
     * @param string[] $files
     */
    protected function duplicateFilesThatHaveNoMatchingMinOrSourceFile(array $files)
    {
        foreach ($files as $file) {
            
            if (preg_match(self::UNMINIFIED_FILE_NAME_PATTERN, $file, $result)) {
                
                //  file only exists as source file
                if ($result[1] === '') {
                    
                    $destination = preg_replace(self::UNMINIFIED_FILE_NAME_PATTERN, '.min.$2', $file);
                } //  File only exists as .min file
                else {
                    
                    $destination = preg_replace(self::UNMINIFIED_FILE_NAME_PATTERN, '.$2', $file);
                }
                
                copy($file, $destination);
            }
        }
    }
    
    
    /**
     * @param ThemeId                     $themeId
     * @param ThemeDirectoryInterface     $directory
     * @param ThemeDirectoryRootInterface $destination
     */
    protected function _copyMainThemeDirectory(
        ThemeId $themeId,
        ThemeDirectoryInterface $directory,
        ThemeDirectoryRootInterface $destination
    ) {
        $childDirectories = $directory->getChildren();
        
        if ($childDirectories) {
            foreach ($childDirectories as $childDir) {
                $this->_copyMainThemeDirectory($themeId, $childDir, $destination);
            }
        }
        
        foreach ($directory->getFiles() as $file) {
            $themeName       = $themeId->getId();
            $sourcePath      = $this->_getRelative($directory->getRoot());
            $destinationPath = $this->_getRelative($destination) . $this->_stringAfter($themeName, $sourcePath);
            
            $sourceFile      = $sourcePath . DIRECTORY_SEPARATOR . $file;
            $destinationFile = $destinationPath . DIRECTORY_SEPARATOR . $file;
            
            try {
                $this->filesystem->copy($sourceFile, $destinationFile);
            } catch (FileNotFoundException $e) {
            }
        }
    }
    
    
    /**
     * @param ThemeId                     $themeId
     * @param ThemeDirectoryInterface     $directory
     * @param ThemeDirectoryRootInterface $destination
     */
    protected function _copyMainThemeStyleEditDirectory(
        ThemeId $themeId,
        ThemeDirectoryInterface $directory,
        ThemeDirectoryRootInterface $destination
    ) {
        $childDirectories = $directory->getChildren();
        
        if ($childDirectories) {
            foreach ($childDirectories as $childDir) {
                $this->_copyMainThemeDirectory($themeId, $childDir, $destination);
            }
        }
        
        foreach ($directory->getFiles() as $file) {
            // ignores files that are created by styleedit
            if (!$themeId->equals($this->sourceThemeId)
                && $this->_endsWith($directory->getRoot()->getPath(), 'styleedit')
                && $this->_endsWith($file, '.json')) {
                continue;
            }
            
            $themeName       = $themeId->getId();
            $sourcePath      = $this->_getRelative($directory->getRoot());
            $destinationPath = $this->_getRelative($destination) . $this->_stringAfter($themeName, $sourcePath);
            
            $sourceFile      = $sourcePath . DIRECTORY_SEPARATOR . $file;
            $destinationFile = $destinationPath . DIRECTORY_SEPARATOR . $file;
            
            try {
                $this->filesystem->copy($sourceFile, $destinationFile);
            } catch (FileNotFoundException $e) {
            }
        }
    }
    
    
    /**
     * @param ThemeDirectoryRootInterface $destination
     *
     * @return ConfigurationCollection
     * @throws Exception
     */
    protected function _loadActiveVariants(ThemeDirectoryRootInterface $destination): ConfigurationCollection
    {
        
        $settingsFilename = $this->_getRelative($destination) . DIRECTORY_SEPARATOR . 'settings.json';
        if ($this->filesystem->has($settingsFilename)) {
            $settingsContent = stream_get_contents($this->filesystem->readStream($settingsFilename));
            $configurations  = json_decode($settingsContent, false);
        } else {
            $configurations = [];
        }
        $activeVariants = [];
        foreach ($configurations as $configuration) {
            if ($configuration->type === 'variant') {
                $activeVariants[] = $configuration;
            }
        }
        
        return ConfigurationCollection::createFromJsonList($activeVariants);
    }
    
    
    /**
     * @param VariantDirectories          $variant
     * @param ThemeDirectoryRootInterface $destination
     *
     * @return bool|CustomThemeCopyResponse|mixed
     * @throws FileNotFoundException
     */
    protected function _overloadVariantDirectory(VariantDirectories $variant, ThemeDirectoryRootInterface $destination)
    {
        $this->_copyThemeDirectory($variant->getId(), $variant->getHtml(), $destination);
        $result = $this->_copyCustomHtmlDirectory($variant, $variant->getCustomHtml(), $destination);
        
        $this->_copyThemeDirectory($variant->getId(), $variant->getJs(), $destination);
        $this->_copyCustomJsDirectory($variant->getId(), $variant->getCustomJs(), $destination);
        
        $this->_copyThemeDirectory($variant->getId(), $variant->getStyles(), $destination);
        $this->_copyCustomStylesDirectory($variant->getId(), $variant->getCustomStyles(), $destination);
        
        $this->_copyThemeImagesDirectory($variant->getId(), $variant->getImages(), $destination);
        
        $this->_copyThemeDirectory($variant->getId(), $variant->getJsExtensions(), $destination);
        
        return $result;
    }
    
    
    /**
     * @param ThemeId                      $themeId
     * @param                              $directory
     * @param ThemeDirectoryRootInterface  $destination
     *
     * @throws FileNotFoundException
     */
    protected function _copyThemeDirectory(
        ThemeId $themeId,
        $directory,
        ThemeDirectoryRootInterface $destination
    ) {
        if ($directory instanceof ThemeDirectoryInterface) {
            $childDirectories = $directory->getChildren();
            if ($childDirectories) {
                foreach ($childDirectories as $childDirectory) {
                    $this->_copyThemeDirectory($themeId, $childDirectory, $destination);
                }
            }
            
            foreach ($directory->getFiles() as $file) {
                $themeName       = $themeId->getId();
                $sourcePath      = $this->_getRelative($directory->getRoot());
                $destinationPath = $this->_getRelative($destination) . $this->_stringAfter($themeName, $sourcePath);
                $destinationPath = $this->replaceCustomInPath($destinationPath);
                
                $sourceFile      = $sourcePath . DIRECTORY_SEPARATOR . $file;
                $destinationFile = $destinationPath . DIRECTORY_SEPARATOR . $file;
                
                try {
                    $this->filesystem->copy($sourceFile, $destinationFile);
                } catch (FileExistsException $e) {
                    $this->filesystem->delete($destinationFile);
                    $this->filesystem->copy($sourceFile, $destinationFile);
                } catch (FileNotFoundException $e) {
                }
            }
        }
    }
    
    
    /**
     * @param IdentifiedThemeInterface    $theme
     * @param                             $directory
     * @param ThemeDirectoryRootInterface $destination
     *
     * @return bool|CustomThemeCopyResponse|mixed
     * @throws FileNotFoundException
     */
    protected function _copyCustomHtmlDirectory(
        IdentifiedThemeInterface $theme,
        $directory,
        ThemeDirectoryRootInterface $destination
    ) {
        $overloadFilesList = [];
        $whitelist         = [];
        
        if ($directory instanceof ThemeDirectoryInterface) {
            $childDirectories = $directory->getChildren();
            if ($childDirectories) {
                $innerWhitelist         = [];
                $innerOverloadFilesList = [];
                
                foreach ($childDirectories as $childDirectory) {
                    $innerResult              = $this->_copyCustomHtmlDirectory($theme, $childDirectory, $destination);
                    $innerWhitelist[]         = $innerResult->getWhiteList()->getArray();
                    $innerOverloadFilesList[] = $innerResult->getOverloaders()->getArray();
                }
                $overloadFilesList = array_merge(...$innerOverloadFilesList);
                $whitelist         = array_merge(...$innerWhitelist);
            }
            
            $themeName       = $theme->getId()->getId();
            $sourcePath      = $this->_getRelative($directory->getRoot());
            $destinationPath = $this->_getRelative($destination) . $this->_stringAfter($themeName, $sourcePath);
            $destinationPath = $this->replaceCustomInPath($destinationPath);
            
            foreach ($directory->getFiles() as $file) {
                $sourceFile      = $sourcePath . DIRECTORY_SEPARATOR . $file;
                $destinationFile = $destinationPath . DIRECTORY_SEPARATOR . $file;
                
                if ($this->filesystem->has($destinationFile)) {
                    $whitelist[] = $this->_extendHtmlFile($sourceFile,
                                                          $destinationFile,
                                                          $theme->getPrefix());
                } elseif ($theme instanceof VariantDirectoriesInterface) {
                    //variants should copy files that is created
                    $this->filesystem->copy($sourceFile, $destinationFile);
                    $whitelist[] = new ExistingFile(new NonEmptyStringType($this->shopSource . DIRECTORY_SEPARATOR
                                                                           . $destinationFile));
                } else {
                    $overloadFilesList[] = new ExistingFile(new NonEmptyStringType($this->shopSource . $sourceFile));
                }
            }
            
            return CustomThemeCopyResponse::createWithOverloadsAndWhitelist(new ExistingFileCollection($overloadFilesList),
                                                                            new ExistingFileCollection($whitelist));
        }
    }
    
    
    /**
     * @param ThemeId                     $themeId
     * @param                             $directory
     * @param ThemeDirectoryRootInterface $destination
     *
     * @throws FileNotFoundException
     */
    protected function _copyCustomJsDirectory(ThemeId $themeId, $directory, ThemeDirectoryRootInterface $destination)
    {
        if ($directory instanceof ThemeDirectoryInterface) {
            $childDirectories = $directory->getChildren();
            if ($childDirectories) {
                foreach ($childDirectories as $childDirectory) {
                    $this->_copyCustomJsDirectory($themeId, $childDirectory, $destination);
                }
            }
            
            foreach ($directory->getFiles() as $file) {
                $themeName       = $themeId->getId();
                $sourcePath      = $this->_getRelative($directory->getRoot());
                $destinationPath = $this->_getRelative($destination) . $this->_stringAfter($themeName, $sourcePath);
                $destinationPath = $this->replaceCustomInPath($destinationPath);
                
                $sourceFile      = $sourcePath . DIRECTORY_SEPARATOR . $file;
                $destinationFile = $this->moveJsFromCustomRootDirectoryToCustomGlobalDirectory($destinationPath
                                                                                               . DIRECTORY_SEPARATOR
                                                                                               . $file,
                                                                                               $sourceFile);
                
                try {
                    $this->filesystem->copy($sourceFile, $destinationFile);
                } catch (FileExistsException $e) {
                    $this->_appendFileContent($destinationFile, $sourceFile);
                } catch (FileNotFoundException $e) {
                }
            }
        }
    }
    
    
    /**
     * @param ThemeId                     $themeId
     * @param                             $directory
     * @param ThemeDirectoryRootInterface $destination
     *
     * @throws FileNotFoundException
     */
    protected function _copyCustomStylesDirectory(
        ThemeId $themeId,
        $directory,
        ThemeDirectoryRootInterface $destination
    ) {
        if ($directory instanceof ThemeDirectoryInterface) {
            $childDirectories = $directory->getChildren();
            if ($childDirectories) {
                foreach ($childDirectories as $childDirectory) {
                    $this->_copyCustomStylesDirectory($themeId, $childDirectory, $destination);
                }
            }
            
            foreach ($directory->getFiles() as $file) {
                $themeName       = $themeId->getId();
                $sourcePath      = $this->_getRelative($directory->getRoot());
                $destinationPath = $this->_getRelative($destination) . $this->_stringAfter($themeName, $sourcePath);
                $destinationPath = $this->replaceCustomInPath($destinationPath);
                
                $sourceFile      = $sourcePath . DIRECTORY_SEPARATOR . $file;
                $destinationFile = $destinationPath . DIRECTORY_SEPARATOR . $file;
                
                if (!$this->filesystem->has($destinationFile)) {
                    $destinationFile = $destinationPath . DIRECTORY_SEPARATOR . 'custom' . DIRECTORY_SEPARATOR
                                       . '_usermod.scss';
                }
                
                try {
                    $this->filesystem->copy($sourceFile, $destinationFile);
                } catch (FileExistsException $e) {
                    $this->_appendFileContent($destinationFile, $sourceFile);
                } catch (FileNotFoundException $e) {
                }
            }
        }
    }
    
    
    /**
     * @param ThemeId                      $themeId
     * @param                              $imagesDirectory
     * @param ThemeDirectoryRootInterface  $destination
     *
     * @throws FileNotFoundException
     */
    protected function _copyThemeImagesDirectory(
        ThemeId $themeId,
        $imagesDirectory,
        ThemeDirectoryRootInterface $destination
    ) {
        if ($imagesDirectory instanceof ThemeDirectoryInterface) {
            $childDirectories = $imagesDirectory->getChildren();
            if ($childDirectories) {
                foreach ($childDirectories as $childDirectory) {
                    $this->_copyThemeImagesDirectory($themeId, $childDirectory, $destination);
                }
            }
            
            foreach ($imagesDirectory->getFiles() as $file) {
                $themeName       = $themeId->getId();
                $sourcePath      = $this->_getRelative($imagesDirectory->getRoot());
                $destinationPath = $this->_getRelative($destination) . $this->_stringAfter($themeName, $sourcePath);
                
                if (!$themeId->equals($this->sourceThemeId)) {
                    $destinationPath = $this->replaceCustomInPath($destinationPath);
                }
                
                $sourceFile      = $sourcePath . DIRECTORY_SEPARATOR . $file;
                $destinationFile = $destinationPath . DIRECTORY_SEPARATOR . $file;
                
                try {
                    $this->filesystem->copy($sourceFile, $destinationFile);
                } catch (FileExistsException $e) {
                    $this->filesystem->delete($destinationFile);
                    $this->filesystem->copy($sourceFile, $destinationFile);
                } catch (FileNotFoundException $e) {
                }
            }
        }
    }
    
    
    /**
     * @param ThemeId                      $themeId
     * @param                              $directory
     * @param ThemeDirectoryRootInterface  $destination
     *
     * @throws FileNotFoundException
     */
    protected function _copyStyleEditThemeDirectory(
        ThemeId $themeId,
        $directory,
        ThemeDirectoryRootInterface $destination
    ) {
        if ($directory instanceof ThemeDirectoryInterface) {
            $childDirectories = $directory->getChildren();
            if ($childDirectories) {
                foreach ($childDirectories as $childDirectory) {
                    $this->_copyThemeDirectory($themeId, $childDirectory, $destination);
                }
            }
            
            foreach ($directory->getFiles() as $file) {
                // ignores files that are created by styleedit
                if ($this->_endsWith($directory->getRoot()->getPath(), 'styleedit') && $this->_endsWith($file, '.json')
                    && !$themeId->equals($this->sourceThemeId)) {
                    continue;
                }
                
                $themeName       = $themeId->getId();
                $sourcePath      = $this->_getRelative($directory->getRoot());
                $destinationPath = $this->_getRelative($destination) . $this->_stringAfter($themeName, $sourcePath);
                $destinationPath = $this->replaceCustomInPath($destinationPath);
                
                $sourceFile      = $sourcePath . DIRECTORY_SEPARATOR . $file;
                $destinationFile = $destinationPath . DIRECTORY_SEPARATOR . $file;
                
                try {
                    $this->filesystem->copy($sourceFile, $destinationFile);
                } catch (FileExistsException $e) {
                    $this->filesystem->delete($destinationFile);
                    $this->filesystem->copy($sourceFile, $destinationFile);
                } catch (FileNotFoundException $e) {
                }
            }
        }
    }
    
    
    /**
     * @param ExistingFile $file
     *
     * @return string
     */
    public function getRelativeFromFile(ExistingFile $file)
    {
        return str_replace($this->shopSource, '', $file->getFilePath());
    }
    
    
    /**
     * @param $sourceFile
     * @param $destinationFile
     * @param $suffix
     *
     * @return mixed
     * @throws FileNotFoundException
     */
    protected function _extendHtmlFile($sourceFile, $destinationFile, $suffix)
    {
        
        $fileExtension = '.' . end(explode('.', $destinationFile));
        
        $renamed     = str_replace($fileExtension, '_' . $suffix . $fileExtension, $destinationFile);
        $newFileName = end(explode(DIRECTORY_SEPARATOR, $renamed));
        
        $this->filesystem->rename($destinationFile, $renamed);
        $this->filesystem->copy($sourceFile, $destinationFile);
        
        $extends = sprintf('{extends file="get_usermod:{$tpl_path}%s"}', $newFileName);
        
        $resource = $this->filesystem->readStream($destinationFile);
        $content  = $extends . PHP_EOL . PHP_EOL . stream_get_contents($resource);
        
        $putStream = tmpfile();
        fwrite($putStream, $content);
        rewind($putStream);
        
        $this->filesystem->putStream($destinationFile, $putStream);
        
        if (is_resource($putStream)) {
            fclose($putStream);
        }
        
        return new ExistingFile(new NonEmptyStringType($this->shopSource . $renamed));
    }
    
    
    /**
     * @param $searchFor
     * @param $subject
     *
     * @return bool|string
     */
    protected function _stringAfter($searchFor, $subject)
    {
        return substr($subject, strpos($subject, $searchFor) + strlen($searchFor));
    }
    
    
    /**
     * @param $haystack
     * @param $needle
     *
     * @return bool
     */
    protected function _endsWith($haystack, $needle)
    {
        $length = strlen($needle);
        if ($length == 0) {
            return true;
        }
        
        return (substr($haystack, -$length) === $needle);
    }
    
    
    /**
     * @param string $path
     *
     * @return string
     */
    protected function replaceCustomInPath($path)
    {
        $pat = '#\/variants\/.*custom#';
        
        if (preg_match($pat, $path) === 0) {
            return $this->_stringReplaceFirst('custom', 'system', $path);
        } else {
            
            return $path;
        }
    }
    
    
    /**
     * Modifies the path if a file would be placed inside the root of the
     * custom directory. The file will be moved inside the global directory of
     * system directory
     *
     * This method is only called for child themes
     *
     * @param string $path
     * @param string $source
     *
     * @return string mixed
     */
    protected function moveJsFromCustomRootDirectoryToCustomGlobalDirectory($path, $source)
    {
        $fileIsInCustomRootDirectoryPattern = '#custom/([^/]+\.js)$#';
        
        //  detecting if the source file is in the root of the custom directory
        if (preg_match($fileIsInCustomRootDirectoryPattern, $source) !== 0) {
            
            //  SHOP_ROOT ends with slash and paths starts with a slash
            $publicPath = SHOP_ROOT . substr($path, 1);
            
            //  Prohibiting changing of paths that already exist in the root of the system directory
            if (!file_exists($publicPath)) {
                
                $fileInSystemRootDirectoryPattern = '#system/([^/]+\.js)$#';
                
                //  the path for the directory has already been changed in a
                //  previous method without viewing the file name
                return preg_replace($fileInSystemRootDirectoryPattern, 'system/Global/$1', $path);
            }
        }
        
        return $path;
    }
    
    
    /**
     * @param $destinationFile
     * @param $sourceFile
     *
     * @throws FileNotFoundException
     */
    protected function _appendFileContent($destinationFile, $sourceFile)
    {
        $resource = $this->filesystem->readStream($destinationFile);
        
        $content   = stream_get_contents($resource) . PHP_EOL . $this->filesystem->read($sourceFile);
        $putStream = tmpfile();
        fwrite($putStream, $content);
        rewind($putStream);
        
        $this->filesystem->putStream($destinationFile, $putStream);
        
        if (is_resource($putStream)) {
            fclose($putStream);
        }
    }
    
    
    /**
     * @param $search
     * @param $replace
     * @param $subject
     *
     * @return string
     */
    protected function _stringReplaceFirst($search, $replace, $subject)
    {
        $pos = strpos($subject, $search);
        if ($pos !== false) {
            return substr($subject, 0, $pos) . $replace . substr($subject, $pos + strlen($search));
        }
        
        return $subject;
    }
    
    
    /**
     * @param ThemeId                      $themeId
     * @param                              $fontsDirectory
     * @param ThemeDirectoryRootInterface  $destination
     *
     * @throws FileNotFoundException
     */
    protected function _copyThemeFontsDirectory(
        ThemeId $themeId,
        $fontsDirectory,
        ThemeDirectoryRootInterface $destination
    ) {
        if ($fontsDirectory instanceof ThemeDirectoryInterface) {
            $childDirectories = $fontsDirectory->getChildren();
            if ($childDirectories) {
                foreach ($childDirectories as $childDirectory) {
                    $this->_copyThemeFontsDirectory($themeId, $childDirectory, $destination);
                }
            }
            
            foreach ($fontsDirectory->getFiles() as $file) {
                $themeName       = $themeId->getId();
                $sourcePath      = $this->_getRelative($fontsDirectory->getRoot());
                $destinationPath = $this->_getRelative($destination) . $this->_stringAfter($themeName, $sourcePath);
                
                $sourceFile      = $sourcePath . DIRECTORY_SEPARATOR . $file;
                $destinationFile = $destinationPath . DIRECTORY_SEPARATOR . $file;
                
                try {
                    $this->filesystem->copy($sourceFile, $destinationFile);
                } catch (FileExistsException $e) {
                    $this->filesystem->delete($destinationFile);
                    $this->filesystem->copy($sourceFile, $destinationFile);
                } catch (FileNotFoundException $e) {
                }
            }
        }
    }
    
}
