<?php
/**
 * Created by PhpStorm.
 * User: j.fischer
 * Date: 19.03.19
 * Time: 08:15
 */

namespace Gambio\StyleEdit\Core\Components\Theme;

use Exception;
use Gambio\StyleEdit\Core\Components\BasicController;
use Gambio\StyleEdit\Core\Components\Theme\Entities\PreviewThemeSettings;
use Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfiguration;
use Gambio\StyleEdit\Core\SingletonPrototype;
use Gambio\StyleEdit\Core\TranslatedException;
use ReflectionException;

/**
 * Class ThemeController
 * @package Gambio\StyleEdit\Core\Components\Theme
 */
class ThemeController extends BasicController
{
    /**
     * @param array $uri
     *
     * @return mixed|void
     * @throws Exception
     */
    public function get(array $uri)
    {
        if (count($uri) > 2) {
            if ($this->theme()) {
                $this->outputTheme();
            } else {
                throw new TranslatedException('StyleEdit.exceptions.invalid-theme-id', [end($uri)], 404);
            }
        } else {
            $this->outputThemeList();
        }
    }
    
    
    /**
     * @throws Exception
     */
    protected function outputTheme()
    {
        $this->outputJson($this->themeService()->getConfigurationById($this->theme()->id())->toArray());
    }
    
    
    /**
     * @throws Exception
     */
    protected function outputThemeList()
    {
        $themeCollection  = $this->getThemesList();
        $gambioThemes     = [];
        $duplicatedThemes = [];
        foreach ($themeCollection as $theme) {
            /** @var ThemeConfiguration $theme */
            
            if ($theme->author() === "Gambio GmbH" && !preg_match('/\-\s(Copy|Kopie)$/', $theme->title())) {
                $gambioThemes[] = $theme->toArray();
            } else {
                $duplicatedThemes[] = $theme->toArray();
            }
        }
        //put the gambio themes inthe begning of the list
        $themeList = array_merge($gambioThemes, $duplicatedThemes);
        
        $this->outputJson($this->themeService()->sortByActive($themeList));
    }
    
    
    /**
     * Save some theme
     *
     * @param array $uri
     * @param       $data
     *
     * @return mixed|void
     * @throws Exception
     */
    public function put(array $uri, $data)
    {
        $this->themeService()->save($data);
    }
    
    
    /** Duplicate a theme
     *
     * @param array $uri
     * @param       $data
     *
     * @return mixed|void
     * @throws Exception
     */
    public function post(array $uri, $data)
    {
        //Ex: styleEdit/Theme/HoneyGrid/duplicate
        if ($this->theme() && count($uri) >= 4 && strtoupper($uri[3]) === 'DUPLICATE') {
            $jsonObject = json_decode($data, false);
            
            if (isset($jsonObject->id)) {
                unset($jsonObject->id);
            }
            
            $configFile = $this->themeService()->duplicateTheme($this->theme(), $jsonObject);
            $this->updateThemeList();
            $this->outputJson($this->getThemesList()->get($configFile->id())->toArray());
        } elseif (count($uri) >= 4 && strtoupper($uri[3]) === 'CREATEPREVIEW') {
            if (!$this->theme()) {
                throw new TranslatedException('StyleEdit.exceptions.invalid-theme-id', [end($uri)], 404);
            }
            
            if ($this->theme()->preview()) {
                throw new TranslatedException('StyleEdit.exceptions.cant-create-preview-of-preview',
                                              [$this->theme()->id()],
                                              404);
            }
            
            $previewThemeId = $this->theme()->id() . '_preview';
            if ($this->getThemesList()->hasKey($previewThemeId)) {
                throw (new TranslatedException('StyleEdit.exceptions.preview-already-exists',
                                               [$this->theme()->id()],
                                               100))->withHttpStatusCode(500);
            }
            
            $jsonObject = (object)[
                'title'   => $this->theme()->title() . ' Preview',
                'id'      => $previewThemeId,
                'preview' => true
            ];
            
            $configFile = $this->themeService()->duplicateTheme($this->theme(), $jsonObject);

            $this->themeService()->patch($configFile, ['extends' => $this->theme()->extendsOf()]);

            $this->updateThemeList();
            $path            = $this->themeService()->createPreviewFolder();
            $previewSettings = new PreviewThemeSettings($previewThemeId, $path, 'templates_c/temp/' . $previewThemeId);
            $this->previewSettingsService()->save($previewSettings);
            $this->outputJson(['id' => $previewThemeId, 'path' => $path]);
        }
    }
    
    
    /**
     * @param array $uri
     * @param       $data
     *
     * @return mixed|void
     * @throws Exception
     */
    public function delete(array $uri, $data)
    {
        if ($this->theme()) {
            $this->themeService()->deleteTheme($this->theme());
        } else {
            throw new Exception('Invalid theme');
        }
    }
    
    
    /**
     * @param array $uri
     * @param       $data
     *
     * @return mixed|void
     * @throws Exception
     */
    public function patch(array $uri, $data)
    {
        if ($this->theme()) {
            $data = json_decode($data, false);
            $this->themeService()->patch($this->theme(), $data);
        }
    }
    
    
    /**
     * @param $id
     *
     * @return ThemeConfiguration
     * @throws TranslatedException
     * @throws ReflectionException
     */
    protected function getTheme($id)
    {
        return $this->themeService()->getConfigurationById($id);
    }
    
    
    /**
     * @return BasicController|void
     */
    public function __clone()
    {
        
    }
}