<?php
/* --------------------------------------------------------------
  SettingsService.php 2019-06-11
  Gambio GmbH
  http://www.gambio.de
  Copyright (c) 2019 Gambio GmbH
  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]
  --------------------------------------------------------------*/

namespace Gambio\StyleEdit\Core\Services;

use Exception;
use Gambio\StyleEdit\Core\Repositories\ConfigurationRepository;
use Gambio\StyleEdit\Core\Repositories\Entities\Configuration;
use Gambio\StyleEdit\Core\Repositories\Entities\ConfigurationCollection;

/**
 * Class SettingsService
 */
class SettingsService
{
    /**
     * @var string
     */
    protected $currentTheme;
    
    /**
     * @var string
     */
    protected $errorMessage = '';
    
    /**
     * @var int
     */
    protected $boxPosition = 1;
    
    /**
     * @var ConfigurationCollection
     */
    protected $configurationsList;
    
    
    /**
     * SettingsService constructor.
     *
     * @param string $currentTheme active theme
     *
     * @throws Exception
     */
    public function __construct(string $currentTheme)
    {
        $this->currentTheme = $currentTheme;
    }
    
    
    /**
     * @return ConfigurationCollection
     * @throws Exception
     */
    public function configurationsList()
    {
        if ($this->configurationsList === null) {
            $rep                      = ConfigurationRepository::createForTheme($this->currentTheme);
            $this->configurationsList = $rep->hasConfiguration() ? $rep->configurationsList() : ConfigurationCollection::createFromJsonList([]);
        }
        
        return $this->configurationsList;
    }
    
    
    /**
     * @param $p_group
     *
     * @return array
     * @throws Exception
     */
    public function getScssByGroup($p_group): array
    {
        $result = [];
        
        /** @var  Configuration $item */
        foreach ($this->configurationsList()->getIterator() as $item) {
            
            $group = $item->group();
            $value = $item->value();
            
            if ($group !== null && $group === $p_group
                && (is_string($value) || is_bool($value) || is_object($value))) {
                
                if (is_string($value) && $value !== '0' && empty(trim($value))) {
                    continue;
                }
                
                switch ($item->type()) {
                    case 'url':
                    case 'imageupload':
                        $result[$item->id()] = '\'' . $value . '\'';
                        break;
                    default:
                        $result[$item->id()] = $value;
                }
            }
        }
        
        return $result;
    }
    
    
    /**
     * Used by StyleEditReaderWrapper
     */
    
    /**
     * Get Sass Code by group and style name
     *
     * @param string $p_group
     *
     * @return string
     * @throws Exception
     */
    public function getScss($p_group): string
    {
        $scss = '';
        
        $group = $this->getScssByGroup($p_group);
        
        if (count($group) > 0) {
            
            foreach ($group as $variableName => $variableValue) {
                
                if (is_string($variableValue)) {
                    
                    $scss .= '$' . $variableName . ': ' . $variableValue . ';' . PHP_EOL;
                } elseif (is_bool($variableValue)) {
                    
                    $scss .= '$' . $variableName . ': ' . ($variableValue === true ? 'true' : 'false') . ';' . PHP_EOL;
                }
            }
        }
        
        return $scss;
    }
    
    
    /**
     * Returns the custom styles from a JSON File
     *
     * @return string
     * @throws Exception
     */
    public function getCustomStyles(): string
    {
        $scss  = '';
        $group = $this->getScssByGroup('custom');
        
        if (count($group) > 0) {
            
            foreach ($group as $variableName => $variableValue) {
                $scss .= $variableValue . PHP_EOL;
            }
        }
        
        return $scss;
    }
    
    
    /**
     * Searches for a setting value identified by its name. If no result is found, null will be returned.
     *
     * @param $p_settingName
     *
     * @return mixed|null
     * @throws Exception
     */
    public function findSettingValueByName($p_settingName)
    {
        /** @var  Configuration $item */
        foreach ($this->configurationsList()->getIterator() as $item) {
            
            if ($item->id() === $p_settingName) {
                
                return $item->value();
            }
        }
        
        return null;
    }
    
    
    /**
     * Returns error message
     *
     * @return string
     */
    public function getErrorMessage(): string
    {
        return $this->errorMessage;
    }
}